@@ -1,17 +1,14 @@
 import os
 import operator
 import re
-import six
 from abc import ABCMeta, abstractmethod
 
-
-from .environments import PIPENV_INSTALL_TIMEOUT
-from .vendor import attr, delegator
-from .utils import find_windows_executable
+from pipenv.vendor import attr
+from pipenv.utils import find_windows_executable, subprocess_run
 
 
 @attr.s
-class Version(object):
+class Version:
 
     major = attr.ib()
     minor = attr.ib()
@@ -29,7 +26,7 @@ class Version(object):
         """
         match = re.match(r'^(\d+)\.(\d+)(?:\.(\d+))?$', name)
         if not match:
-            raise ValueError('invalid version name {0!r}'.format(name))
+            raise ValueError(f'invalid version name {name!r}')
         major = int(match.group(1))
         minor = int(match.group(2))
         patch = match.group(3)
@@ -59,17 +56,16 @@ class InstallerNotFound(RuntimeError):
 
 class InstallerError(RuntimeError):
     def __init__(self, desc, c):
-        super(InstallerError, self).__init__(desc)
-        self.out = c.out
-        self.err = c.err
+        super().__init__(desc)
+        self.out = c.stdout
+        self.err = c.stderr
 
 
-@six.add_metaclass(ABCMeta)
-class Installer(object):
+class Installer(metaclass=ABCMeta):
 
-    def __init__(self):
+    def __init__(self, project):
         self.cmd = self._find_installer()
-        super(Installer, self).__init__()
+        self.project = project
 
     def __str__(self):
         return self.__class__.__name__
@@ -109,22 +105,21 @@ class Installer(object):
             # Check for explicitly set install locations (e.g. PYENV_ROOT, ASDF_DIR).
             os.path.join(os.path.expanduser(os.getenv(env_var, '/dev/null')), 'bin', name),
             # Check the pyenv/asdf-recommended from-source install locations
-            os.path.join(os.path.expanduser('~/.{}'.format(name)), 'bin', name),
+            os.path.join(os.path.expanduser(f'~/.{name}'), 'bin', name),
         ):
             if candidate is not None and os.path.isfile(candidate) and os.access(candidate, os.X_OK):
                 return candidate
         raise InstallerNotFound()
 
     def _run(self, *args, **kwargs):
-        timeout = kwargs.pop('timeout', delegator.TIMEOUT)
+        timeout = kwargs.pop('timeout', 30)
         if kwargs:
             k = list(kwargs.keys())[0]
-            raise TypeError('unexpected keyword argument {0!r}'.format(k))
+            raise TypeError(f'unexpected keyword argument {k!r}')
         args = (self.cmd,) + tuple(args)
-        c = delegator.run(args, block=False, timeout=timeout)
-        c.block()
-        if c.return_code != 0:
-            raise InstallerError('failed to run {0}'.format(args), c)
+        c = subprocess_run(args, timeout=timeout)
+        if c.returncode != 0:
+            raise InstallerError(f'failed to run {args}', c)
         return c
 
     @abstractmethod
@@ -149,7 +144,7 @@ class Installer(object):
             ), key=operator.attrgetter('cmpkey'))
         except ValueError:
             raise ValueError(
-                'no installable version found for {0!r}'.format(name),
+                f'no installable version found for {name!r}',
             )
         return best_match
 
@@ -174,7 +169,7 @@ class Pyenv(Installer):
     def iter_installable_versions(self):
         """Iterate through CPython versions available for Pipenv to install.
         """
-        for name in self._run('install', '--list').out.splitlines():
+        for name in self._run('install', '--list').stdout.splitlines():
             try:
                 version = Version.parse(name.strip())
             except ValueError:
@@ -190,7 +185,7 @@ class Pyenv(Installer):
         """
         c = self._run(
             'install', '-s', str(version),
-            timeout=PIPENV_INSTALL_TIMEOUT,
+            timeout=self.project.s.PIPENV_INSTALL_TIMEOUT,
         )
         return c
 
@@ -203,7 +198,7 @@ class Asdf(Installer):
     def iter_installable_versions(self):
         """Iterate through CPython versions available for asdf to install.
         """
-        for name in self._run('list-all', 'python').out.splitlines():
+        for name in self._run('list-all', 'python').stdout.splitlines():
             try:
                 version = Version.parse(name.strip())
             except ValueError:
@@ -219,6 +214,6 @@ class Asdf(Installer):
         """
         c = self._run(
             'install', 'python', str(version),
-            timeout=PIPENV_INSTALL_TIMEOUT,
+            timeout=self.project.s.PIPENV_INSTALL_TIMEOUT,
         )
         return c
