@@ -1,7 +1,5 @@
-# The following comment should be removed at some point in the future.
-# mypy: disallow-untyped-defs=False
-
-from __future__ import absolute_import
+from optparse import Values
+from typing import List
 
 from pipenv.patched.notpip._internal.cli.base_command import Command
 from pipenv.patched.notpip._internal.cli.status_codes import SUCCESS
@@ -15,9 +13,11 @@ class HelpCommand(Command):
       %prog <command>"""
     ignore_require_venv = True
 
-    def run(self, options, args):
+    def run(self, options: Values, args: List[str]) -> int:
         from pipenv.patched.notpip._internal.commands import (
-            commands_dict, create_command, get_similar_commands,
+            commands_dict,
+            create_command,
+            get_similar_commands,
         )
 
         try:
@@ -29,9 +29,9 @@ class HelpCommand(Command):
         if cmd_name not in commands_dict:
             guess = get_similar_commands(cmd_name)
 
-            msg = ['unknown command "%s"' % cmd_name]
+            msg = [f'unknown command "{cmd_name}"']
             if guess:
-                msg.append('maybe you meant "%s"' % guess)
+                msg.append(f'maybe you meant "{guess}"')
 
             raise CommandError(' - '.join(msg))
 
