@@ -1,7 +1,4 @@
-from pipenv.patched.notpip._internal.utils.typing import MYPY_CHECK_RUNNING
-
-if MYPY_CHECK_RUNNING:
-    from typing import Optional, List
+from typing import List, Optional
 
 
 def main(args=None):
