@@ -1,22 +1,25 @@
-# The following comment should be removed at some point in the future.
-# mypy: disallow-untyped-defs=False
-
-from __future__ import absolute_import
-
 import logging
 import os
 import re
+from typing import List, Optional, Tuple
 
-from pipenv.patched.notpip._internal.utils.logging import indent_log
 from pipenv.patched.notpip._internal.utils.misc import (
+    HiddenText,
     display_path,
     is_console_interactive,
-    rmtree,
+    is_installable_dir,
     split_auth_from_netloc,
 )
-from pipenv.patched.notpip._internal.utils.subprocess import make_command
-from pipenv.patched.notpip._internal.utils.typing import MYPY_CHECK_RUNNING
-from pipenv.patched.notpip._internal.vcs.versioncontrol import VersionControl, vcs
+from pipenv.patched.notpip._internal.utils.subprocess import CommandArgs, make_command
+from pipenv.patched.notpip._internal.vcs.versioncontrol import (
+    AuthInfo,
+    RemoteNotFoundError,
+    RevOptions,
+    VersionControl,
+    vcs,
+)
+
+logger = logging.getLogger(__name__)
 
 _svn_xml_url_re = re.compile('url="([^"]+)"')
 _svn_rev_re = re.compile(r'committed-rev="(\d+)"')
@@ -24,39 +27,34 @@ _svn_info_xml_rev_re = re.compile(r'\s*revision="(\d+)"')
 _svn_info_xml_url_re = re.compile(r'<url>(.*)</url>')
 
 
-if MYPY_CHECK_RUNNING:
-    from typing import Optional, Tuple
-    from pipenv.patched.notpip._internal.utils.subprocess import CommandArgs
-    from pipenv.patched.notpip._internal.utils.misc import HiddenText
-    from pipenv.patched.notpip._internal.vcs.versioncontrol import AuthInfo, RevOptions
-
-
-logger = logging.getLogger(__name__)
-
-
 class Subversion(VersionControl):
     name = 'svn'
     dirname = '.svn'
     repo_name = 'checkout'
-    schemes = ('svn', 'svn+ssh', 'svn+http', 'svn+https', 'svn+svn')
+    schemes = (
+        'svn+ssh', 'svn+http', 'svn+https', 'svn+svn', 'svn+file'
+    )
 
     @classmethod
     def should_add_vcs_url_prefix(cls, remote_url):
+        # type: (str) -> bool
         return True
 
     @staticmethod
     def get_base_rev_args(rev):
+        # type: (str) -> List[str]
         return ['-r', rev]
 
     @classmethod
     def get_revision(cls, location):
+        # type: (str) -> str
         """
         Return the maximum revision for all files under a given location
         """
         # Note: taken from setuptools.command.egg_info
         revision = 0
 
-        for base, dirs, files in os.walk(location):
+        for base, dirs, _ in os.walk(location):
             if cls.dirname not in dirs:
                 dirs[:] = []
                 continue    # no sense walking uncontrolled subdirs
@@ -69,15 +67,17 @@ class Subversion(VersionControl):
             dirurl, localrev = cls._get_svn_url_rev(base)
 
             if base == location:
+                assert dirurl is not None
                 base = dirurl + '/'   # save the root url
             elif not dirurl or not dirurl.startswith(base):
                 dirs[:] = []
                 continue    # not part of the same svn tree, skip it
             revision = max(revision, localrev)
-        return revision
+        return str(revision)
 
     @classmethod
     def get_netloc_and_auth(cls, netloc, scheme):
+        # type: (str, str) -> Tuple[str, Tuple[Optional[str], Optional[str]]]
         """
         This override allows the auth information to be passed to svn via the
         --username and --password options instead of via the URL.
@@ -85,7 +85,7 @@ class Subversion(VersionControl):
         if scheme == 'ssh':
             # The --username and --password options can't be used for
             # svn+ssh URLs, so keep the auth information in the URL.
-            return super(Subversion, cls).get_netloc_and_auth(netloc, scheme)
+            return super().get_netloc_and_auth(netloc, scheme)
 
         return split_auth_from_netloc(netloc)
 
@@ -93,7 +93,7 @@ class Subversion(VersionControl):
     def get_url_rev_and_auth(cls, url):
         # type: (str) -> Tuple[str, Optional[str], AuthInfo]
         # hotfix the URL scheme after removing svn+ from svn+ssh:// readd it
-        url, rev, user_pass = super(Subversion, cls).get_url_rev_and_auth(url)
+        url, rev, user_pass = super().get_url_rev_and_auth(url)
         if url.startswith('ssh://'):
             url = 'svn+' + url
         return url, rev, user_pass
@@ -111,27 +111,32 @@ class Subversion(VersionControl):
 
     @classmethod
     def get_remote_url(cls, location):
-        # In cases where the source is in a subdirectory, not alongside
-        # setup.py we have to look up in the location until we find a real
-        # setup.py
+        # type: (str) -> str
+        # In cases where the source is in a subdirectory, we have to look up in
+        # the location until we find a valid project root.
         orig_location = location
-        while not os.path.exists(os.path.join(location, 'setup.py')):
+        while not is_installable_dir(location):
             last_location = location
             location = os.path.dirname(location)
             if location == last_location:
                 # We've traversed up to the root of the filesystem without
-                # finding setup.py
+                # finding a Python project.
                 logger.warning(
-                    "Could not find setup.py for directory %s (tried all "
+                    "Could not find Python project for directory %s (tried all "
                     "parent directories)",
                     orig_location,
                 )
-                return None
+                raise RemoteNotFoundError
 
-        return cls._get_svn_url_rev(location)[0]
+        url, _rev = cls._get_svn_url_rev(location)
+        if url is None:
+            raise RemoteNotFoundError
+
+        return url
 
     @classmethod
     def _get_svn_url_rev(cls, location):
+        # type: (str) -> Tuple[Optional[str], int]
         from pipenv.patched.notpip._internal.exceptions import InstallationError
 
         entries_path = os.path.join(location, cls.dirname, 'entries')
@@ -141,17 +146,18 @@ class Subversion(VersionControl):
         else:  # subversion >= 1.7 does not have the 'entries' file
             data = ''
 
+        url = None
         if (data.startswith('8') or
                 data.startswith('9') or
                 data.startswith('10')):
-            data = list(map(str.splitlines, data.split('\n\x0c\n')))
-            del data[0][0]  # get rid of the '8'
-            url = data[0][3]
-            revs = [int(d[9]) for d in data if len(d) > 9 and d[9]] + [0]
+            entries = list(map(str.splitlines, data.split('\n\x0c\n')))
+            del entries[0][0]  # get rid of the '8'
+            url = entries[0][3]
+            revs = [int(d[9]) for d in entries if len(d) > 9 and d[9]] + [0]
         elif data.startswith('<?xml'):
             match = _svn_xml_url_re.search(data)
             if not match:
-                raise ValueError('Badly formatted data: %r' % data)
+                raise ValueError(f'Badly formatted data: {data!r}')
             url = match.group(1)    # get repository URL
             revs = [int(m.group(1)) for m in _svn_rev_re.finditer(data)] + [0]
         else:
@@ -165,8 +171,11 @@ class Subversion(VersionControl):
                 xml = cls.run_command(
                     ['info', '--xml', location],
                     show_stdout=False,
+                    stdout_only=True,
                 )
-                url = _svn_info_xml_url_re.search(xml).group(1)
+                match = _svn_info_xml_url_re.search(xml)
+                assert match is not None
+                url = match.group(1)
                 revs = [
                     int(m.group(1)) for m in _svn_info_xml_rev_re.finditer(xml)
                 ]
@@ -182,6 +191,7 @@ class Subversion(VersionControl):
 
     @classmethod
     def is_commit_id_equal(cls, dest, name):
+        # type: (str, Optional[str]) -> bool
         """Always assume the versions don't match"""
         return False
 
@@ -198,7 +208,7 @@ class Subversion(VersionControl):
         #   Empty tuple: Could not parse version.
         self._vcs_version = None  # type: Optional[Tuple[int, ...]]
 
-        super(Subversion, self).__init__()
+        super().__init__()
 
     def call_vcs_version(self):
         # type: () -> Tuple[int, ...]
@@ -213,13 +223,17 @@ class Subversion(VersionControl):
         #      compiled Feb 25 2019, 14:20:39 on x86_64-apple-darwin17.0.0
         #   svn, version 1.7.14 (r1542130)
         #      compiled Mar 28 2018, 08:49:13 on x86_64-pc-linux-gnu
+        #   svn, version 1.12.0-SlikSvn (SlikSvn/1.12.0)
+        #      compiled May 28 2019, 13:44:56 on x86_64-microsoft-windows6.2
         version_prefix = 'svn, version '
-        version = self.run_command(['--version'], show_stdout=False)
+        version = self.run_command(
+            ['--version'], show_stdout=False, stdout_only=True
+        )
         if not version.startswith(version_prefix):
             return ()
 
         version = version[len(version_prefix):].split()[0]
-        version_list = version.split('.')
+        version_list = version.partition('-')[0].split('.')
         try:
             parsed_version = tuple(map(int, version_list))
         except ValueError:
@@ -256,7 +270,6 @@ class Subversion(VersionControl):
         in this class.
 
             - checkout
-            - export
             - switch
             - update
 
@@ -281,23 +294,6 @@ class Subversion(VersionControl):
 
         return []
 
-    def export(self, location, url):
-        # type: (str, HiddenText) -> None
-        """Export the svn repository at the url to the destination location"""
-        url, rev_options = self.get_url_rev_options(url)
-
-        logger.info('Exporting svn repository %s to %s', url, location)
-        with indent_log():
-            if os.path.exists(location):
-                # Subversion doesn't like to check out over an existing
-                # directory --force fixes this, but was only added in svn 1.5
-                rmtree(location)
-            cmd_args = make_command(
-                'export', self.get_remote_call_options(),
-                rev_options.to_args(), url, location,
-            )
-            self.run_command(cmd_args, show_stdout=False)
-
     def fetch_new(self, dest, url, rev_options):
         # type: (str, HiddenText, RevOptions) -> None
         rev_display = rev_options.to_display()
