@@ -13,8 +13,8 @@ See <http://github.com/ActiveState/appdirs> for details and usage.
 # - Mac OS X: http://developer.apple.com/documentation/MacOSX/Conceptual/BPFileSystem/index.html
 # - XDG spec for Un*x: http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html
 
-__version_info__ = (1, 4, 3)
-__version__ = '.'.join(map(str, __version_info__))
+__version__ = "1.4.4"
+__version_info__ = tuple(int(segment) for segment in __version__.split("."))
 
 
 import sys
@@ -38,7 +38,7 @@ if sys.platform.startswith('java'):
         # *sys.platform* style strings.
         system = 'linux2'
 elif sys.platform == 'cli' and os.name == 'nt':
-    # Detect Windows in IronPython to match pip._internal.utils.compat.WINDOWS
+    # Detect Windows in IronPython to match pipenv.patched.notpip._internal.utils.compat.WINDOWS
     # Discussion: <https://github.com/pypa/pip/pull/7501>
     system = 'win32'
 else:
@@ -92,10 +92,6 @@ def user_data_dir(appname=None, appauthor=None, version=None, roaming=False):
         path = os.path.expanduser('~/Library/Application Support/')
         if appname:
             path = os.path.join(path, appname)
-        if not os.path.isdir(path):
-            path = os.path.expanduser('~/.config/')
-            if appname:
-                path = os.path.join(path, appname)
     else:
         path = os.getenv('XDG_DATA_HOME', os.path.expanduser("~/.local/share"))
         if appname:
@@ -257,8 +253,6 @@ def site_config_dir(appname=None, appauthor=None, version=None, multipath=False)
             if version:
                 appname = os.path.join(appname, version)
             pathlist = [os.path.join(x, appname) for x in pathlist]
-        # always look in /etc directly as well
-        pathlist.append('/etc')
 
         if multipath:
             path = os.pathsep.join(pathlist)
